from pathlib import Path

from . import config as C, db, shell


def bindir(path):
    d = path / 'bin'
    if d.is_dir():
        return d
    if C.ON_WINDOWS:
        d = path / 'Scripts'
        if d.is_dir():
            return d
    return None


def check(path):
    return bindir(path) is not None


def list_available():
    for l in C.VENV_DIR.glob('*'):
        if check(l):
            yield l.name


def load(path):
    print('LOADING', path)
    bd = bindir(path)
    if bd is None:
        raise Exception('how did you get here??')
    l, p = shell.get_shell()
    l(bd, p)


def bind(name, path=None):
    if path is None:
        path = Path.cwd()
    C.setup_conf()
    venv_path = C.VENV_DIR / name
    dbi = db.DB()
    path_str = str(path)
    if path_str in dbi:
        raise Exception(f'{path} is already bound to {path_str}')
    dbi[path_str] = name
    dbi.dump()
    return venv_path


def for_path(path=None):
    if path is None:
        path = Path.cwd()
    else:
        path = path.absolute()
    root = Path(path.root)
    while path != root:
        r = db.DB().get(str(path))
        if r is not None:
            return C.VENV_DIR / r
        path = path.parent
    return None


from . import __description__, __version__
import argparse
import sys
import os
import platform
import json
from pathlib import Path

from .config import ON_WINDOWS, VENV_DIR, CONF_DIR, ENV_DB
from . import db, venv


def parseargs(args):
    parser = argparse.ArgumentParser(
        add_help=False,
        description=__description__,
        usage='%(prog)s [OPTIONS] ...',
    )
    parser.add_argument('-h', '--help', action='help', help=argparse.SUPPRESS)
    parser.add_argument('-l', '--list', action='store_true', help='list available venvs')
    parser.add_argument('-b', '--bind', type=str, metavar='ENV', help='bind venv to current path')
    parser.add_argument('--version', action='version', version=__version__)

    return parser.parse_args(args)



class CommandError(Exception):
    pass


def main(args=None):
    args = parseargs(args)
    if args.list:
        for _e in venv.list_available():
            print(_e)
        return 0

    if args.bind:
        env = venv.bind(args.bind)
        if env is None:
            print(f'env {args.bind} does not exist. Use -c instead to create it or -l to see the ones available')
            return 1
        venv.load(env)

    env = venv.for_path()
    if env is None:
        print('No env configured for current directory. Use "-b ENV" to bind an existing one or "-c ENV" to create and bind to a new one')
        return 1
    
    venv.load(env)


if __name__ == '__main__':
    exit(main())
